<?php

namespace Drupal\week\Plugin\views\filter;

use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\filter\NumericFilter;

/**
 *
 *
 * @ViewsFilter("week")
 */
class Week extends NumericFilter {

  /**
   * {@inheritdoc}
   */
  protected function valueForm(&$form, FormStateInterface $form_state) {
    parent::valueForm($form, $form_state);
    // Adds a simple option to use "@current".
    // TODO: For exposed form this will need a widget.
    $form['value']['value']['#description'] = $this->t('To use the current week set value to "@current"');
    $form['value']['min']['#description'] = $this->t('To use the current week set value to "@current"');
    $form['value']['max']['#description'] = $this->t('To use the current week set value to "@current"');
  }

  /**
   * {@inheritdoc}
   */
  protected function opBetween($field) {
    //select * from node__field_week_test where CONVERT(REPLACE(field_week_test_value, '-', ''), SIGNED) > 202105
    if (is_numeric($this->value['min']) && is_numeric($this->value['max'])) {
      $operator = $this->operator == 'between' ? 'BETWEEN' : 'NOT BETWEEN';
      $field = "CONVERT(REPLACE($field, '-', ''), SIGNED) {$operator} {[$this->value['min'], $this->value['max']]}";
      $this->query->addWhere($this->options['group'], $field);
    }
    elseif (is_numeric($this->value['min'])) {
      $operator = $this->operator == 'between' ? '>=' : '<';
      $field = "CONVERT(REPLACE($field, '-', ''), SIGNED) {$operator} {$this->value['min']}";
      $this->query->addWhere($this->options['group'], $field, $this->value['min'], $operator);
    }
    elseif (is_numeric($this->value['max'])) {
      $operator = $this->operator == 'between' ? '<=' : '>';
      $field = "CONVERT(REPLACE($field, '-', ''), SIGNED) {$operator} {$this->value['max']}";
      $this->query->addWhere($this->options['group'], $field, $this->value['max'], $operator);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function query() {
    $this->convertValue();
    parent::query(); // TODO: Change the autogenerated stub
  }

  /**
   * {@inheritdoc}
   */
  protected function opSimple($field) {
    $field = "CONVERT(REPLACE($field, '-', ''), SIGNED) {$this->operator} {$this->value['value']}";
    $this->query->addWhereExpression($this->options['group'], $field);
  }

  /**
   * Simple helper to convert entered values if set to @current.
   */
  private function convertValue() {
    foreach ($this->value as $key => &$value) {
      if ($value === '@current') {
        $value = date('Y') . date('W');
      }
    }
  }
}

